﻿using HarmonyLib;
using Steamworks;
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AzureSky;
using System.Threading.Tasks;

namespace SolarPanelV2 {
    public class SolarPanelObject : MonoBehaviour_ID_Network, IRaycastable {
        private CanvasHelper canvas;
        private Network_Player localPlayer;
        private bool showingText;
        public Battery battery;
        private MeshRenderer batteryMeshRenderer;
        public bool placed = false;

        public DateTime lastRecharge = DateTime.Now;
        public int secsBetweenRecharges = 5;
        public AzureSkyController skyController;

        public int batteryMaxUses;

        void Start() {
            lastRecharge = DateTime.Now;
            Debug.Log("Start was called!");

            battery = GetComponentInChildren<Battery>();
            batteryMaxUses = ItemManager.GetItemByName("Battery").MaxUses;
            batteryMeshRenderer = battery.GetComponentInChildren<MeshRenderer>();
            batteryMeshRenderer.enabled = true;

            InvokeRepeating("UpdateLateClients", 5, 5);
        }

        void UpdateLateClients() {
            if (!placed) { return; }
            // Used to update clients that join late, this will force a battery sync event to occur.
            if (Raft_Network.IsHost && !battery.BatterySlotIsEmpty) {
                RAPI.GetLocalPlayer().Network.RPC(new Message_Battery(Messages.Battery_Update, RAPI.GetLocalPlayer().Network.NetworkIDManager, RAPI.GetLocalPlayer().steamID, this.ObjectIndex, battery.GetBatteryInstance().Uses), Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }
        }

        void Update() {
            if (!placed) { return; }
            if (skyController == null) {
                skyController = FindObjectOfType<AzureSkyController>();
            }

            if (!Raft_Network.IsHost || skyController.timeOfDay.hour < 5 || skyController.timeOfDay.hour > 19 || battery.BatterySlotIsEmpty)
            {
                return;
            }

            if (battery.NormalizedBatteryLeft < 1f && lastRecharge.AddSeconds(secsBetweenRecharges) <= DateTime.Now)
            {
                lastRecharge = DateTime.Now;
                battery.AddBatteryUsesNetworked(1);
            }
        }

        public async void OnIsRayed() {
            if (!placed) { return; }
            while (this.canvas == null) {
                this.canvas = ComponentManager<CanvasHelper>.Value;
                await Task.Delay(1);
            }
            while (this.localPlayer == null) {
                this.localPlayer = RAPI.GetLocalPlayer();
                await Task.Delay(1);
            }
            if (CanvasHelper.ActiveMenu != MenuType.None || !Helper.LocalPlayerIsWithinDistance(base.transform.position, Player.UseDistance)) {
                if (this.showingText) {
                    this.canvas.displayTextManager.HideDisplayTexts();
                    this.showingText = false;
                }
                return;
            }
            if (skyController.timeOfDay.hour < 5 || skyController.timeOfDay.hour > 19) {
                this.canvas.displayTextManager.ShowText("Its the spooky night! The solar panel won't work!", 0, true, 0);
                this.showingText = true;
                return;
            }
            if (!battery.BatterySlotIsEmpty) {
                if (battery.NormalizedBatteryLeft < 1f) {
                    this.canvas.displayTextManager.ShowText("The battery is charging... (" + Mathf.Round(battery.NormalizedBatteryLeft * 100) + "%)", 0, true, 0);
                    this.showingText = true;
                } else {
                    this.canvas.displayTextManager.ShowText("The battery is fully charged!", 0, true, 0);
                    this.showingText = true;
                }
            } else {
                this.canvas.displayTextManager.ShowText("No battery, place one to charge it!", 0, true, 0);
                this.showingText = true;
                return;
            }
        }

        public void OnRayEnter() {
        }

        void IRaycastable.OnRayExit() {
            if (this.showingText) {
                this.canvas.displayTextManager.HideDisplayTexts();
                this.showingText = false;
            }
        }

        public override bool Deserialize(Message_NetworkBehaviour msg, CSteamID remoteID) {
            Messages type = msg.Type;
            switch (type) {
                case Messages.Battery_Insert:
                case Messages.Battery_Take:
                case Messages.Battery_Update:
                case Messages.Battery_OnOff:
                    return this.battery.OnBatteryMessage(msg);
            }
            return base.Deserialize(msg, remoteID);
        }

        protected override void OnDestroy() {
            NetworkIDManager.RemoveNetworkID(this);
            base.OnDestroy();
        }

        public void OnBlockPlaced() {
            placed = true;
            NetworkIDManager.AddNetworkID(this);
        }
    }
}
